<?php
// FILE DOWNLOAD INDEX
// Display directory contents with links to download files.
// Created by subpixel (subpixel@subpixels.com)
// Zip contents processing from somewhere else
//
// FEATURES
// - Optional downloadextensions.txt file to specify what files are listed (by file extension)
// - Sizes of downloads displayed in bytes, KB, MB or GB
// - Optional title.txt file contents used for title
// - Optional description.txt file contents displayed at top
// - Image carousel for images and image list
// - Image carousel for video thumbnails (only if one or more thumbs found) and video list
// - Subdirectories list
// - Zip files list (with contents of each zip files)
// - Miscellaneous (other) files list
// - Optional licence.txt file contents displayed at bottom

// Path of this file
$startPath = dirname(__FILE__);

// Name of the start path (without parent paths)
$startBasename = basename($startPath);

// Parent path
$startParentPath = dirname($startPath);

// Title text (optional file)

$titleText = getTextFileContents("title.txt", $startPath);
if (!isset($titleText)) {
	$titleText = "Downloads from subpixel";
}

// Description text (optional file)
$descText = getTextFileContents("description.txt", $startPath);

// Licence text (optional)
$licText = getTextFileContents("licence.txt", $startPath);

// Extensions list file name
$extensionsFileName = "downloadextensions.txt";

// Default extensions
$downloadExtensions = array("bmp", "jpg", "png", "pdf", "avi", "mov", "mpg", "mp4", "m4a", "mp3", "wav", "aiff", "ogg", "zip", "7z", "rar");

// Image extensions
$imageExtensions = array("bmp", "jpg", "png");

// Video extensions
$videoExtensions = array("avi", "mov", "mpg", "mp4", "m4v");

// Look for extensions in configuration file
$extensionsPath = getTextPath($extensionsFileName, $startPath);
if (is_file($extensionsPath)) {
	$downloadExtensions = file($extensionsPath, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
}

// Get list of folders and filenames (with sizes), with separate list for zips
$folders = array(); // Value is folder name
$fileSizes = array(); // Key is filename, value is file size (in bytes)
$miscFiles = array(); // Value is filename
$images = array(); // Value is image filename
$imageThumbs = array(); // Key is image filename, value is thumbnail filename
$videos = array(); // Value is video filename
$videoThumbs = array(); // Value is thumbnail filename
$zips = array(); // Value is zip filename

// Keep track of whether Galleria is required
$galleriaRequired = false;

// Examine contents of the directory
foreach (new DirectoryIterator($startPath) as $downloadDI) {
	// Ignore "." and ".."
	if (!$downloadDI->isDot()) {
		if ($downloadDI->isFile()) {
			$downloadFilename = $downloadDI->getFilename();
			
			// Only show if the filename has an extension in the list of download extensions
			$dotPos = strrpos($downloadFilename, ".");
			if ($dotPos !== false) {
				$downloadExtension = strtolower( substr($downloadFilename, $dotPos + 1)  );
			
                // Note that only files with extensions listed in $downloadExtensions are listed,
                // which means image and video files can be excluded
				if (in_array($downloadExtension, $downloadExtensions)) {
					$fileSize = $downloadDI->getSize();
					$fileSizes[$downloadFilename] = $fileSize;
					
                    // Separate into categories for special processing
					if ($downloadExtension === "zip") {
						$zips[] = $downloadFilename;
					} else if (in_array($downloadExtension, $imageExtensions)) {
					   $images[] = $downloadFilename;
					} else if (in_array($downloadExtension, $videoExtensions)) {
						$videos[] = $downloadFilename;
					} else {
						$miscFiles[] = $downloadFilename;
					}
				}
			}
		}
		else if ($downloadDI->isDir()) {
			$foldername = $downloadDI->getFilename();
			$folders[] = $foldername;
		}
	}
}

// Videos: Look for video thumbnails; only display carousel if at least one thumbnail found
if (count($videos) > 0) {
	sort($videos);

	$hasVideoThumbs = false;

	foreach ($videos as $video) {
		$dotPos = strrpos($video, ".");
		if ($dotPos !== false) {
			$videoName = substr($video, 0, $dotPos);

			$thumbnail = "missing-thumbnail.jpg";

			$candidates = array($videoName . ".jpg", $videoName . ".JPG", $videoName . "T01.JPG");

			foreach ($candidates as $candidate) {

				// Check for video thumbnail
				$thumbnailPath = $startParentPath . DIRECTORY_SEPARATOR . "thumbnails" . DIRECTORY_SEPARATOR . $candidate;

				if (is_file($thumbnailPath)) {
					$thumbnail = $candidate;
					$hasVideoThumbs = true;
					break;
				}
			}

			$videoThumbs[] = $thumbnail;
		}
	}

	// Remove dummy thumbnails if no proper thumbnails found, else require Galleria to display carousel
	if (!$hasVideoThumbs) {
		$videoThumbs = array();
	} else {
		$galleriaRequired = true;
	}
}

// Images: Look for image thumbnails. Always display carousel.
if (count($images) > 0) {
	sort($images);

	$hasImgThumbs = false;

	foreach ($images as $image) {
		$dotPos = strrpos($image, ".");
		if ($dotPos !== false) {
			$imageName = substr($image, 0, $dotPos);

			$thumbnail = "missing-thumbnail.jpg";

			$candidates = array($image, $imageName . ".jpg", $imageName . ".JPG");

			foreach ($candidates as $candidate) {

				// Check for image thumbnail
				$thumbnailPath = $startParentPath . DIRECTORY_SEPARATOR . "thm" . DIRECTORY_SEPARATOR . $candidate;

				if (is_file($thumbnailPath)) {
					$thumbnail = $candidate;
					$hasImgThumbs = true;
					break;
				}
			}

			// Store thumbnail, indexed by image filename
			$imageThumbs[$image] = $thumbnail;
		}
	}

	// Galleria required
	$galleriaRequired = true;
}

?>
<html>
<head>
<title><?= $titleText ?> - <?= $startBasename ?> | subpixels.com</title>
<?php
if ($galleriaRequired) {
?>
<script src="/lib/jquery/jquery-3.3.1.min.js"></script>
<script src="/lib/galleria/galleria-1.5.7.min.js"></script>
<?php
}
?>
<style type="text/css">
h1, h2, h3, h4, h5, h6, div, p, li
{
	font-family: arial,sans-serif;
}

h1, h2, h3, h4, h5, h6 { margin: 0; padding: 0; }

h1 + div, h2 + div, h3 + div { margin-top: 0.5em; }

div + h3 { margin-top: 1em; }

ul { margin: 0; padding: 0 0 0 1em; }

img { vertical-align: middle; }

.group
{
	margin-top: 1em;
	padding: 0.5em;
	background: #eee;
	border: 1px solid black;
}

.licencetext
{
}

.descriptiontext
{
	margin: 1em 0;
}

li
{
	margin: 0.25em 0;
}

.downloads > li { font-size: small; }
.zipcontents { font-size: small; }

.zipcontents > input[type=checkbox] ~ ul
{
    display: none;
    visibility: hidden;
    -webkit-transition: .125s linear;
    -moz-transition: .125s linear;
    -ms-transition: .125s linear;
    -o-transition: .125s linear;
    transition: .125s linear;
}

.zipcontents > input[type=checkbox]:checked ~ ul
{
    display: block;
    visibility: visible;
    width: 100%;
}

.galleria
{
	max-width: 800px;
	height: 600px;
	background: #000
}

</style>
</head>
<body>
<h1><?= $titleText ?></h1>
<div class="downloadslisting">

<?php
// Description: Display description snippet
if (isset($descText)) {
?>
	<div class="group">
		<div class="descriptiontext">
			<?= $descText ?>
		</div>
	</div>
<?php
}
?>

<?php							
// Images: Container for image carousel and image list
if (count($images) > 0) {
?>
	<div class="group">
<?php
	// Images: Display image thumbnail carousel (if any thumbnails)
	if (count($imgThumbs) > 0) {
?>
		<h3>Image Gallery</h3>
		<div class="galleria">
<?php
		foreach ($images as $image) {
			$thumbnail = $imageThumbs[$image];
?>
			<a href="<?= $image ?>"><img src="../thm/<?= $thumbnail ?>" /></a>
<?php
		}
?>
        </div>
		
<?php
	}
?>
 		<h3>Image Files</h3>
		<ol class="downloads">
<?php
	$totalSize = 0;

	foreach ($images as $file) {
		$size = $fileSizes[$file];
		$totalSize += $size;
?>
			<li><a href="<?= $file ?>"><?= $file ?></a> - <?= dispSize($size) ?></li>
<?php
	}
?>
		</ol>

		<p>Total images size: <?= dispSize($totalSize) ?></p>
	</div>
<?php
}
?>

<?php
// Videos: Container for video thumbnail carousel and video list
if (count($videos) > 0) {
?>
	<div class="group">
<?php
	// Videos: Display video thumbnail carousel (if any thumbnails)
	if (count($videoThumbs) > 0) {
?>
		<h3>Video Gallery</h3>
		<div class="galleria">
<?php
		foreach ($videoThumbs as $thumbnail) {
?>
			<a href="../thumbnails/<?= $thumbnail ?>"><img src="../thm/<?= $thumbnail ?>" /></a>
<?php
		}
?>
		</div>
		
<?php
	}
?>
		<h3>Video Files</h3>
		<ol class="downloads">
<?php
	$totalSize = 0;

	foreach ($videos as $file) {
		$size = $fileSizes[$file];
		$totalSize += $size;
?>
			<li><a href="<?= $file ?>"><?= $file ?></a> - <?= dispSize($size) ?></li>
<?php
	}
?>
		</ol>

		<p>Total videos size: <?= dispSize($totalSize) ?></p>
	</div>
<?php
}
?>

<?php
// Initialise Galleria
if ($galleriaRequired) {
?>
<script>
(function() {
    Galleria.loadTheme("/lib/galleria/themes/classic/galleria.classic.min.js");
    Galleria.run(".galleria");
}());
</script>
<?php
}
?>

<?php
// Folders: Display folder list
if (count($folders) > 0) {
	sort($folders);
?>
	<div class="group">
		<h3>Folders</h3>
		<ul class="folders">
<?php
	foreach ($folders as $folder) {
?>
			<li><a href="<?= $folder ?>"><?= $folder ?></a></li>
<?php
	}
?>
		</ul>
	</div>
<?php
}
?>

<?php
// Zips: Display zip list
if (count($zips) > 0) {
	sort($zips);
	$totalSize = 0;
?>
	<div class="group">
		<h3>Zips</h3>
		<ol class="zips">
<?php
	foreach ($zips as $file) {
		$size = $fileSizes[$file];
		$totalSize += $size;
?>
			<li><a href="<?= $file ?>"><?= $file ?></a> - <?= dispSize($size) ?>
<?php
		$zip = new ZipArchive;
		$zipPath = $startPath . DIRECTORY_SEPARATOR . $file;
		$res = $zip->open($zipPath, ZipArchive::CHECKCONS);

		if ($res === TRUE) {
			if ($zip->numFiles > 0) {
?>
			<br/>
			<label class="zipcontents"><input type="checkbox" role="button" /><span>Show Contents</span>
				<ul>
<?php
				for ($i = 0; $i < $zip->numFiles; $i++) {
					$stat = $zip->statIndex($i);
					$filename = $stat["name"];
					$size = $stat["size"];
					$basename = basename($filename);
					$info = "";
					if (substr($filename, -1) === "/") {
						$info = " (folder)";
					}
					else {
						$info = " - " . dispSize($size);
					}
?>
					<li><?= $basename ?><?= $info ?></li>
<?php
				}
?>
				</ul>
			</label>
<?php
			}
			
			$zip->close();
		}
		else {
			switch($res){
				case ZipArchive::ER_EXISTS:
					$ErrMsg = "File already exists.";
					break;

				case ZipArchive::ER_INCONS:
					$ErrMsg = "Zip archive inconsistent.";
					break;
				   
				case ZipArchive::ER_MEMORY:
					$ErrMsg = "Malloc failure.";
					break;
				   
				case ZipArchive::ER_NOENT:
					$ErrMsg = "No such file.";
					break;
				   
				case ZipArchive::ER_NOZIP:
					$ErrMsg = "Not a zip archive.";
					break;
				   
				case ZipArchive::ER_OPEN:
					$ErrMsg = "Can't open file.";
					break;
				   
				case ZipArchive::ER_READ:
					$ErrMsg = "Read error.";
					break;
				   
				case ZipArchive::ER_SEEK:
					$ErrMsg = "Seek error.";
					break;
			   
				default:
					$ErrMsg = "Unknown (Code $res)";
					break;
			}
?>
				Error opening zip: <?= $ErrMsg ?>
<?php
		}
?>
			</li>
<?php
	}
?>
		</ol>

		<p>Total zips size: <?= dispSize($totalSize) ?></p>
	</div>
<?php
}
?>

<?php
// Misc files: Display miscellaneous file list
if (count($miscFiles) > 0) {
	sort($miscFiles);
	$totalSize = 0;
?>
	<div class="group">
 		<h3>Files</h3>
		<ol class="downloads">
<?php
	foreach ($miscFiles as $file) {
		$size = $fileSizes[$file];
		$totalSize += $size;
?>
          <li><a href="<?= $file ?>"><?= $file ?></a> - <?= dispSize($size) ?></li>
<?php
	}
?>
        </ol>

        <p>Total files size: <?= dispSize($totalSize) ?></p>
	</div>
<?php
}
?>

<?php
// Licence text: display
if (isset($licText)) {
?>
	<div class="group">
		<div class="licencetext">
<?= $licText ?>
		</div>
	</div>
<?php
}
?>

	<div class="group">
		<p>Extensions: <?= join(", ", $downloadExtensions) ?></p>
	</div>
</body>
</html>
<?php
// Get the path of the text file in the start directory or the parent directory, NULL if not found
function getTextPath($fileName, $startPath) {
	$textPath = NULL;
	$filePath = $startPath . DIRECTORY_SEPARATOR . $fileName;

	if (is_file($filePath)) {
		$textPath = $filePath;
	}
	else {
		$filePath = dirname($startPath) . DIRECTORY_SEPARATOR . $fileName;

		if (is_file($filePath)) {
			$textPath = $filePath;
		}
	}

	return $textPath;
}

// Get the contents of the text file in the start directory or the parent directory
function getTextFileContents($fileName, $startPath) {
	$contents = NULL;
	$textPath = getTextPath($fileName, $startPath);
	
	if (isset($textPath)) {
		$contents = file_get_contents($textPath);
	}
	
	return $contents;
}

// Return a user-friendly string for a file size
function dispSize($bytes) {
	if ($bytes > 1073741824) return (round($bytes / 10737418.24) / 100) . " GB"; // 2 decimal places
	if ($bytes > 1048576) return (round($bytes / 104857.6) / 10) . " MB"; // 1 decimal place
	if ($bytes > 1024) return (round($bytes / 1024)) . " kB"; // no decimal places
	return ($bytes . " bytes");
}
?>
